/* 
 * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA 02110-1301  USA
 */
#ifndef _MFORMS_TABVIEW_H_
#define _MFORMS_TABVIEW_H_

#include <mforms/view.h>

namespace mforms
{
  class TabView;

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef SWIG
  struct MFORMS_EXPORT TabViewImplPtrs
  {
    bool (__stdcall *create)(TabView*,bool);
    void (__stdcall *set_active_tab)(TabView*,int);
    int (__stdcall *get_active_tab)(TabView*);
    int (__stdcall *add_page)(TabView*,View*,const std::string&);
  };
#endif
#endif

  /** A Notebook/Tabbed view. */
  class MFORMS_EXPORT TabView : public View
  {
    TabViewImplPtrs *_tabview_impl;

    sigc::signal<void> _signal_tab_changed;

  public:
    /** Constructor.
     
     @param tabless - whether the tabview should have no tabs. Switching tabs can then
     be performed programatically only. */
    TabView(bool tabless = false);
    /** Sets the currently selected/active tab */
    void set_active_tab(int index);
    /** Get currently selected tab */
    int get_active_tab();
    /** Add a new page to the tab view, with its tab caption. */
    int add_page(View *page, const std::string& caption);

#ifndef SWIG
    /** Signal emited when the tab is switched by user.
     
     In Python use add_tab_changed_callback()
     */
    sigc::signal<void> signal_tab_changed() { return _signal_tab_changed; }
#endif
  };
};


#endif
